/* verify.c */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "bigdigits.h"
#include "undosig.h"

#define MOD_SIZE 64
#define HEX_MOD_SIZE MOD_SIZE*8
#define SHA1_DIGEST_SIZE	20

int main( int argc, char *argv[] )
{
    FILE * fp;

    unsigned int pubsize = 0;
    unsigned int sigsize = 0;

    unsigned char sig[HEX_MOD_SIZE];
    unsigned char pub[HEX_MOD_SIZE];

	char *res  = NULL;

    char *pub_file;
    char *msg_file;

	/* Force linker to include copyright notice for BigInt library
	   in executable object image */
	copyright_notice();

   /* Command line args */
    if (argc != 3)
    {
	printf( "Usage: verify <pub_key_file> <signed_file>\n" );
	exit( 1 );
    }
    else
    {
	pub_file = argv[1];
	msg_file = argv[2];
    }

    /*========================================
     * Get signed message from a file
     *======================================*/
    fp = fopen( msg_file, "r" );
    if (fp == NULL)
    {
	printf("Couldn't open %s.", msg_file);
	return 0;
    }

    sigsize = fread( sig, sizeof(char), HEX_MOD_SIZE, fp );
    printf( "Read %d bytes from %s.\n", sigsize, msg_file );
	
    /*========================================
     * Set n
     *======================================*/
    fp = fopen( pub_file, "r" );
    if (fp == NULL)
    {
	printf("Couldn't open %s.", pub_file);
	return 0;
    }

    pubsize = fread( pub, sizeof(char), HEX_MOD_SIZE, fp );
    printf( "Read %d bytes from %s.\n\n", pubsize, pub_file );

    res = (char *)malloc( (SHA1_DIGEST_SIZE * 2)+1 );
    memset( res, 0, (SHA1_DIGEST_SIZE * 2)+1 );

    undoSignature( res, pub, sig );

    printf( "\nresult: %s\n", res );

    return 0;
}
