#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <tcpa.h>

/* {{{ int tpm_hash_and_extend() */

int tpm_hash_and_extend(unsigned char *sha1, int pcr) {
   char hash_and_extend_fmt[] = "00 C1 T 00 00 00 14 L %";
   int result;
   char buffer[TCPA_MAX_BUFF_SIZE];
   
   memset(buffer, 0, TCPA_MAX_BUFF_SIZE);
   
   result = buildbuff(hash_and_extend_fmt, buffer,
					  pcr, 20, sha1);

   if(result <= 0) {
	  result = 1;
	  goto out;
   }

   result = TPM_Transmit(buffer, NULL, NULL);

 out:
   return result;
}

/* }}} */

/* {{{ inline unsigned char hex2bin(char input, char *input_txt) */

inline unsigned char hex2bin(char input, char *input_txt) {
   char lcinput = tolower(input);
   char output = 0;
	  
   if(lcinput >= '0' && lcinput <= '9') {
	  output = lcinput - '0';
   } else if(lcinput >= 'a' && lcinput <= 'f') {
	  output = lcinput - 'a' + 10;
   } else {
	  fprintf(stderr, " ERROR, unexpected input character '%c' in input string '%s'.\n",
			  input, input_txt);
   }
   return output;
}

/* }}} */

/* {{{ int main(int argc, char *argv[]) */

int main(int argc, char *argv[]) {
   unsigned char *sha1_txt;
   unsigned char sha1[20];
   int pcr;
   int result;
   int i;
   
   if (argc != 3) {
	  fprintf(stderr, "Usage: hashextend (40 digit sha1) pcr#\n");
	  exit(1);
   }

   sha1_txt = argv[1];
   pcr = atoi(argv[2]);

   for (i = 0; i < 20; i++) {
	  sha1[i] =
		 (hex2bin(sha1_txt[i * 2], sha1_txt) << 4) |
		 (hex2bin(sha1_txt[(i * 2) + 1], sha1_txt) & 0xf);
   }
   
   result = tpm_hash_and_extend(sha1, pcr);

   exit (result);
}

/* }}} */
