#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include "bigdigits.h"

#define MOD_SIZE 64
#define HEX_MOD_SIZE MOD_SIZE*8
#define SHA1_DIGEST_SIZE	20

void dumpMsg( DIGIT_T m[], char *fname, unsigned int len );

int main( int argc, char *argv[] )
{
   int i = 0;
   FILE * fp;

   DIGIT_T digit = 0;

   unsigned int msgsize   = 0;
   unsigned int privsize  = 0;
   unsigned int pubsize   = 0;
   unsigned int numdigits = 0;

   unsigned char msg[SHA1_DIGEST_SIZE * 2];
   unsigned char pub[HEX_MOD_SIZE];
   unsigned char prv[HEX_MOD_SIZE];

   char *ptr  = NULL;
   char *pub_file;
   char *prv_file;
   char *msg_file;
   char *res_file;

   DIGIT_T n[MOD_SIZE], d[MOD_SIZE];
   DIGIT_T m[MOD_SIZE], c[MOD_SIZE];

   /* Force linker to include copyright notice for BigInt library
	  in executable object image */
   copyright_notice();

   /* Command line args */
   if (argc != 5)
	  {
		 printf( "Usage: sign <pub_key_file> <priv_key_file>" \
				 " <message_file> <result_file>\n" );
		 exit( 1 );
	  }
   else
	  {
		 pub_file = argv[1];
		 prv_file = argv[2];
		 msg_file = argv[3];
		 res_file = argv[4];
	  }


   /*========================================
	* Get message from a file
	*======================================*/
   mpSetZero( m, MOD_SIZE );

   fp = fopen( msg_file, "r" );
   if (fp == NULL)
	  {
		 fprintf(stderr, "Couldn't open '%s': ", msg_file);
		 perror("");
		 return 0;
	  }

   msgsize = fread( msg, sizeof(char), SHA1_DIGEST_SIZE * 2, fp );
   printf( "Read %d bytes from %s.\n", msgsize, msg_file );
	
   /* Scan the string into m */
   ptr = &(msg[0]);

   numdigits = msgsize / sizeof(DIGIT_T);
   for (i = 0; i < numdigits; i++)
	  {
		 sscanf( ptr, "%4c", &digit );
		 m[i] = digit;
		 ptr += 4;
	  }

   printf("m = "); mpPrintTrimNL(m, MOD_SIZE);


   /*========================================
	* Get modyulus from a file
	*======================================*/
   mpSetZero(n, MOD_SIZE);

   fp = fopen( pub_file, "r" );
   if (fp == NULL)
	  {
		 fprintf(stderr, "Couldn't open '%s': ", pub_file);
		 perror("");
		 return 0;
	  }

   pubsize = fread( pub, sizeof(char), HEX_MOD_SIZE, fp );
   printf( "\nRead %d bytes from %s.\n", pubsize, pub_file );
	
   /* Scan the string into n */
   ptr = &(pub[0]);

   numdigits = pubsize / sizeof(DIGIT_T);
   for (i = 0; i < numdigits; i++)
	  {
		 sscanf( ptr, "%8x", &digit );
		 n[i] = digit;
		 ptr += 8;
	  }

   printf("n = "); mpPrintNL(n, MOD_SIZE);


   /*========================================
	* Get private key
	*======================================*/
   mpSetZero(d, MOD_SIZE);

   fp = fopen( prv_file, "r" );
   if (fp == NULL)
	  {
		 fprintf(stderr, "Couldn't open '%s': ", prv_file);
		 perror("");
		 return 0;
	  }

   privsize = fread( prv, sizeof(char), HEX_MOD_SIZE, fp );
   printf( "\nRead %d bytes from %s.\n", privsize, prv_file );
	
   /* Scan the string into d */
   ptr = &(prv[0]);

   numdigits = privsize / sizeof(DIGIT_T);
   for (i = 0; i < numdigits; i++)
	  {
		 sscanf( ptr, "%8x", &digit );
		 d[i] = digit;
		 ptr += 8;
	  }

   printf("d = "); mpPrintNL(d, MOD_SIZE);

   /* Sign with private key c = m^d mod n */
   printf( "\nSigning . . .\n" );
   mpModExp(c, m, d, n, MOD_SIZE);
   printf("c = "); mpPrintNL(c, MOD_SIZE);

   dumpMsg( c, res_file, MOD_SIZE );

   return 0;
}


/*=======================================================
 * Dump the msg to fname.
 ======================================================*/
void dumpMsg( DIGIT_T m[], char *fname, unsigned int len )
{
    FILE *fp;
    int i = 0;

    fp = fopen( fname, "w" );
    if (fp == NULL)
    {
	fprintf(stderr, "Couldn't open '%s': ", fname);
	perror("");
	exit( 1 );
    }
    
    while (len--)
    {
	fprintf(fp, "%08lx", m[i]);
	i++;
    }
}
