/*
 * undosig.c - undoes a signature.  duh.
 */


#include "bigdigits.h"

#ifdef __KERNEL__
/* Compiling in the kernel */
#include <linux/ctype.h>
#include <linux/string.h>
#include <linux/kernel.h>
#else
/* Compiling outside the kernel */

/* This define is necessary because -ansi disables inline */
#define inline 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#endif

#define MOD_SIZE 64
#define HEX_MOD_SIZE MOD_SIZE*8
#define SHA1_DIGEST_SIZE 20

static inline unsigned char hex2bin(char input) {
   char lcinput = tolower(input);
   char output = 0;
	  
   if(lcinput >= '0' && lcinput <= '9') {
	  output = lcinput - '0';
   } else if(lcinput >= 'a' && lcinput <= 'f') {
	  output = lcinput - 'a' + 10;
   }

   return output;
}

void undoSignature( char *sha, unsigned char *pub, unsigned char *msg )
{
   DIGIT_T n[MOD_SIZE], e[MOD_SIZE];
   DIGIT_T m[MOD_SIZE], c[MOD_SIZE];
   DIGIT_T digit = 0;

   char ch;
   char *ptr = NULL;    
    
   unsigned int idx = 0;

   /* Force linker to include copyright notice for BigInt library
	  in executable object image */
   copyright_notice();

   
   mpSetZero( c, MOD_SIZE );
   mpSetZero( n, MOD_SIZE );
   mpSetZero( m, MOD_SIZE );

   /* Scan the msg into c */
   ptr = &(msg[0]);

   for (idx = 0; idx < MOD_SIZE; idx++)
	  {
#if 0
		 sscanf( ptr, "%8x", &digit );
#else
		 digit =
			(hex2bin(ptr[0]) << 28) |
			(hex2bin(ptr[1]) << 24) |
			(hex2bin(ptr[2]) << 20) |
			(hex2bin(ptr[3]) << 16) |
			(hex2bin(ptr[4]) << 12) |
			(hex2bin(ptr[5]) << 8) |
			(hex2bin(ptr[6]) << 4) |
			(hex2bin(ptr[7]) << 0);
#endif			
		 c[idx] = digit;
		 ptr += 8;
	  }

   /* printf("c = "); mpPrintNL(c, MOD_SIZE); */


   /* Set e */
   mpSetZero(e, MOD_SIZE);
   e[0] = 0x10001;
   /* printf("\ne = "); mpPrintTrimNL(e, MOD_SIZE); */


   /* Scan the string into n */
   ptr = pub;

   for (idx = 0; idx < MOD_SIZE; idx++)
	  {
#if 0
		 sscanf( ptr, "%8x", &digit );
#else
		 digit =
			(hex2bin(ptr[0]) << 28) |
			(hex2bin(ptr[1]) << 24) |
			(hex2bin(ptr[2]) << 20) |
			(hex2bin(ptr[3]) << 16) |
			(hex2bin(ptr[4]) << 12) |
			(hex2bin(ptr[5]) << 8) |
			(hex2bin(ptr[6]) << 4) |
			(hex2bin(ptr[7]) << 0);
#endif
		 n[idx] = digit;
		 ptr += 8;
	  }

   /* printf("\nn = "); mpPrintNL(n, MOD_SIZE); */


   /* Encrypt with the public key, m=c^e mod n */
   mpModExp(m, c, e, n, MOD_SIZE);
   /* printf("\nm = "); mpPrintTrimNL(m, MOD_SIZE); */


   /* Get the string out */
   ptr = (char *)m;
   for (idx = 0; idx < (SHA1_DIGEST_SIZE * 2); idx++)
	  {
		 sscanf( ptr, "%c", &ch );
		 sha[idx] = ch;
		 ptr++;
	  }
}
