/* $Id: enforcer_lsm.h,v 1.38 2003/11/18 19:06:17 omen Exp $
 *
 * This is the Linux Enforcer Security Module
 *
 * Written and maintained by Omen Wild <Omen.Wild@Dartmouth.EDU>.
 * Feel free to email with questions or concerns.
 *
 * Copyright (C) 2003 Trustees of Dartmouth College
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifndef __ENFORCER_LSM_H
#define __ENFORCER_LSM_H

#include <asm-i386/mach-default/irq_vectors.h>

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/security.h>
#include <linux/stat.h>
#include <linux/skbuff.h>
#include <linux/netlink.h>
#include <linux/ctype.h>
#include <linux/file.h>
#include <linux/spinlock.h>

#include <linux/crypto.h>
#include <linux/blkdev.h>
#include <linux/fs.h>
#include <net/tcp.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/vmalloc.h>
#include <linux/dcache.h>
#include <linux/list.h>
#include <linux/namespace.h>
#include <linux/writeback.h>
#include <linux/quotaops.h>
#include <linux/moduleparam.h>

#include "enforcer_common.h"

#define ENFORCER_HASH_SIZE 8192	/* Number of elements in the hash table */
#define ENFORCER_SIGNATURE_SIZE 512	/* Size of the public key signature on disk */
#define SHA1_DIGEST_SIZE 20

/* Functions available externally */
void enforcer_reverse_buffer(char *buffer, int len);
inline char enforcer_bin2hex(int input);
inline u8 enforcer_hex2bin(char input, char *input_txt);

#ifdef CONFIG_ENFORCER_DEBUG
#define enforcer_print_binary(msg, binary, len) _enforcer_print_binary(__FUNCTION__, msg, binary, len)
inline void _enforcer_print_binary(const char *function, char *msg, u8 *binary, int len);
#endif

typedef enum enforcer_action {
   A_DENY = 1 << 1,
   A_LOG = 1 << 2,
   A_PANIC = 1 << 3,
   A_TPM_LOCK = 1 << 4,
   A_CALL_HELPER = 1 << 5,
} enforcer_action;

struct db_entry {
   char *file;
   u8 sha1[SHA1_DIGEST_SIZE];
   enforcer_action action;
   struct list_head list;
};

#if defined(CONFIG_SECURITY_enforcer_MODULE)
#define MY_NAME THIS_MODULE->name
#else
#define MY_NAME "Enforcer"
#endif

/* {{{ #define printd(level, fmt, arg...) */

#ifdef CONFIG_ENFORCER_DEBUG
#define printd(level, fmt, arg...)					\
   do {												\
      if (enforcer_debug_level >= level)						\
         printk(KERN_INFO "%s:%s:%d: " fmt,			\
				MY_NAME , __FUNCTION__, __LINE__,	\
				## arg);							\
   } while (0)
#else
#define printd(level, fmt, arg...)
#endif

/* }}} */

/* {{{ #define printw(level, fmt, arg...) */

#ifdef CONFIG_ENFORCER_DEBUG
#define printw(level, fmt, arg...)				\
   do {											\
	  printk(level "%s:%s:%d: " fmt,			\
			 MY_NAME , __FUNCTION__, __LINE__,	\
			 ## arg);							\
   } while (0)
#else
#define printw(level, fmt, arg...)				\
   do {											\
	  printk(level "%s: " fmt,					\
			 MY_NAME,							\
			 ## arg);							\
   } while (0)
#endif

/* }}} */

#endif /* __ENFORCER_LSM_H */
