#ifdef CONFIG_ENFORCER_DEBUG
static void enforcer_dump_db_hash (void);
#endif

/* Externally accessable */
inline void enforcer_sha12str(char *sha1_txt, u8 *sha1);
inline void enforcer_str2sha1(u8 *sha1, char *sha1_txt);
inline char enforcer_bin2hex(int input);
inline u8 enforcer_hex2bin(char input, char *input_txt);


/* TCPA related */
int enforcer_tcpa_lock(void);
int enforcer_tcpa_hash_and_extend(u8 *sha1, int pcr_number);
int enforcer_tcpa_unseal(char *blob_filename,
						 char *password_filename,
						 u8 *result, int result_buffer_size,
						 int *tpm_result_len);

/* Internal functions */
static inline int enforcer_atoi(const char *name);

static inline void enforcer_insert_entry_hash (struct db_entry *entry);
static int enforcer_process_database_entry(char *file, int file_len, char *sha1_txt, int sha1_txt_len, char *action, int action_len);
static int enforcer_verify_db_sig(struct file *filp, char *filename);
static int enforcer_read_database(char *filename);
static void enforcer_hash_init(void);
static void enforcer_hash_exit(void);
static int enforcer_read_database_init(char *db);

static inline unsigned int enforcer_hash_calculate(char *filename);
static int enforcer_do_readfile(char *filename, struct crypto_tfm *tfm, struct file *file);
static struct db_entry *enforcer_hash_lookup(char *filename);

static int enforcer_calculate_sha1(u8 *sha1, struct file *file, char *filename);
static int enforcer_verify_sha1(struct db_entry *h, u8 *sha1);
static struct vfsmount *enforcer_find_vfsmount(struct inode *inode, struct list_head **start);

/* LSM Tap points used */
static int enforcer_inode_permission (struct inode *inode, int mask, struct nameidata *nd);
static void enforcer_sb_post_mountroot (void);

/* LSM/Module init/exit */
static int enforcer_register_security (const char *name, struct security_operations *ops);
static int enforcer_unregister_security (const char *name, struct security_operations *ops);
//static int __init enforcer_init (void);
//static void __exit enforcer_exit (void);
