/****************************************************************************/
/*                                                                          */
/*                             OAIP/OSAP protocols                          */
/*                                                                          */
/* This file is copyright 2003 IBM. See "License" for details               */
/****************************************************************************/

#include "tcpa.h"
#include "buildbuff.h"
#include "hmac.h"
#include "oiaposap.h"

#include "../enforcer_lsm.h"
extern int enforcer_debug_level;

/****************************************************************************/
/*                                                                          */
/* Open an OIAP session                                                     */
/*                                                                          */
/****************************************************************************/
uint32_t TPM_OIAP(uint32_t *handle, char *enonce)
{
    unsigned char oiap_open_fmt[] = "00 C1 T 00 00 00 0A";
    unsigned char *tcpadata;
    uint32_t ret;
	int tcpadata_len;

    /* check input arguments */
    if (handle == NULL || enonce == NULL)
	   return -EINVAL;
	   
	tcpadata = kmalloc(TCPA_MAX_BUFF_SIZE, GFP_KERNEL);
	if(!tcpadata) {
	   printk(KERN_ERR MY_NAME ":%s: unable to allocate %d bytes of memory.\n",
			  __FUNCTION__, TCPA_MAX_BUFF_SIZE);
	   return -ENOMEM;
	}
	
    /* build request buffer */
    tcpadata_len = buildbuff(oiap_open_fmt, tcpadata);
    if (tcpadata_len <= 0) {
	   printd(0, "buildbuff(...) = %d\n", tcpadata_len);
	   ret = 1;
	   goto out;
	}
	
    /* transmit request to TPM and get result */
    ret = TPM_Transmit(tcpadata);
    if (ret) {
	   printd(0, "TPM_Transmit(tcpadata) = %d\n", ret);
	   goto out;
	}
	
    *handle = ntohl(*(uint32_t *)(tcpadata+TCPA_DATA_OFFSET));
    memcpy(enonce, &tcpadata[TCPA_DATA_OFFSET + 4], TCPA_NONCE_SIZE);

 out:
	if(tcpadata)
	   kfree(tcpadata);

	return ret;
}

/****************************************************************************/
/*                                                                          */
/* Open an OSAP session                                                     */
/*                                                                          */
/****************************************************************************/
#if 0
uint32_t TPM_OSAP(osapsess *sess, unsigned char *key, uint16_t etype,
                  uint32_t evalue, FILE *log)
{
    unsigned char osap_open_fmt[] = "00 C1 T 00 00 00 0B S L %";
    unsigned char tcpadata[TCPA_MAX_BUFF_SIZE];
    uint32_t ret;

    if (key == NULL)
        return 1;
    RAND_bytes(sess->ononceOSAP, TCPA_NONCE_SIZE);
    ret = buildbuff(osap_open_fmt, tcpadata, etype, evalue,
                    TCPA_NONCE_SIZE, sess->ononceOSAP);
    if (ret <= 0)
        return 1;

    ret = TPM_Transmit(tcpadata, log, "OSAP");
    if (ret)
        return ret;
    sess->handle = ntohl(*(uint32_t *) (tcpadata + TCPA_DATA_OFFSET));
    memcpy(sess->enonce, &(tcpadata[TCPA_DATA_OFFSET + 4]),
           TCPA_NONCE_SIZE);
    memcpy(sess->enonceOSAP,
           &(tcpadata[TCPA_DATA_OFFSET + 4 + TCPA_NONCE_SIZE]),
           TCPA_NONCE_SIZE);
    ret = rawhmac(sess->ssecret, key, TCPA_HASH_SIZE, TCPA_NONCE_SIZE,
                  sess->enonceOSAP, TCPA_NONCE_SIZE, sess->ononceOSAP, 0,
                  0);
    return ret;
}
#endif

/****************************************************************************/
/*                                                                          */
/* Terminate the Handle Opened by TPM_OIAPOpen, or TPM_OSAPOpen             */
/*                                                                          */
/****************************************************************************/
uint32_t TPM_Terminate_Handle(uint32_t handle)
{
    unsigned char hand_close_fmt[] = "00 C1 T 00 00 00 96 L";
    unsigned char *tcpadata;
    uint32_t ret;

	tcpadata = kmalloc(TCPA_MAX_BUFF_SIZE, GFP_KERNEL);
	if(!tcpadata) {
	   printk(KERN_ERR MY_NAME ":%s: unable to allocate %d bytes of memory.\n",
			  __FUNCTION__, TCPA_MAX_BUFF_SIZE);
	   return -ENOMEM;
	}

    ret = buildbuff(hand_close_fmt, tcpadata, handle);
    if (ret <= 0) {
	   ret = 1;
	   goto out;
	}
	
    ret = TPM_Transmit(tcpadata);

 out:
	if(tcpadata)
	   kfree(tcpadata);

	return ret;
}
