/****************************************************************************/
/*                                                                          */
/*  transmit.c - send blobs to/from TPM, with optional debugging            */
/*                                                                          */
/* This file is copyright 2003 IBM. See "License" for details               */
/****************************************************************************/

#include "tcpa.h"

#include "../enforcer_lsm.h"

extern int debug_level;

#if 0
/* optional verbose logging of data to/from tcpa chip */
static void showBlob(unsigned char *blob, char *string, FILE * tcpa_logfp)
{
    uint32_t i, len;

    if (!tcpa_logfp)
        return;
    len = ntohl(*(uint32_t *) & blob[TCPA_PARAMSIZE_OFFSET]);
    fprintf(tcpa_logfp, "        %s length=%d\n        ", string, len);
    for (i = 0; i < len; i++) {
        if (i && !(i % 16)) {
            fprintf(tcpa_logfp, "\n        ");
        }
        fprintf(tcpa_logfp, "%.2X ", blob[i]);
    }
    fprintf(tcpa_logfp, "\n");
    fflush(tcpa_logfp);
}
#endif

uint32_t TPM_Transmit(unsigned char *blob)
{
   int len;
   uint32_t size, ret;
   
   size = ntohl(*(uint32_t *) & blob[TCPA_PARAMSIZE_OFFSET]);

   len = tpm_kernel_write(blob, size);
   if (len <= 0) {
	  printk(KERN_ERR ":%s: TPM write error: errno = %d\n",
			 __FUNCTION__, -len);
	  return len;
   }

   len = tpm_kernel_read(blob, TCPA_MAX_BUFF_SIZE);
   if (len <= 0) {
	  printk(KERN_ERR ":%s: TPM read error: errno = %d\n",
			 __FUNCTION__, -len);
	  return len;
   }

   ret = ntohl(*(uint32_t *) & blob[TCPA_RETURN_OFFSET]);

   return (ret);
}
