/****************************************************************************/
/*                                                                          */
/*                            Seal a Data File                              */
/*                                                                          */
/*  This file is copyright 2003 IBM. See "License" for details              */
/****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <tcpa.h>

#include <buildbuff.h>

#define TPM_TAG_RQU_AUTH2_COMMAND 0x00C3
#define TPM_ORD_CertifyKey 50

int main(int argc, char *argv[]) {
   /* Should be arguments */
   char *certPassword = "tnd0007";
   char *keyPassword = "tnd0007";
   uint32_t certHandle = 0x40ED00;
   uint32_t keyHandle =  0xA62801;


   unsigned char certPasswordHash[TCPA_HASH_SIZE];
   unsigned char keyPasswordHash[TCPA_HASH_SIZE];
   
   FILE *log = stdout;

   int i;

   /* TPM_TAG_RQU_AUTH2_COMMAND */
   unsigned char certify_key[] = "00 C3 T l L L % L % o % L % o %";

   unsigned char tcpadata[TCPA_MAX_BUFF_SIZE];

   unsigned char antiReplay[TCPA_NONCE_SIZE];

   osapsess certAuthHandle;
   unsigned char certAuth[TCPA_HASH_SIZE];

   unsigned char nonceodd[TCPA_NONCE_SIZE];
   
   osapsess keyAuthHandle;
   unsigned char keyAuth[TCPA_HASH_SIZE];

   uint32_t ordinal = htonl(TPM_ORD_CertifyKey);
   
   int ret;

   certAuthHandle.handle = 0xffffffff;
   keyAuthHandle.handle = 0xffffffff;

   sha1(certPassword, strlen(certPassword), certPasswordHash);
   sha1(keyPassword, strlen(keyPassword), keyPasswordHash);

   /* Open OSAP Session */
   ret = TPM_OSAP(&certAuthHandle, certPasswordHash,
				  0x0001,
				  certHandle,
				  log);
   if (ret) {
	  fprintf(stderr, "TPM_OSAP #1 returned %d\n", ret);
	  goto out;
   }

   ret = TPM_OSAP(&keyAuthHandle, keyPasswordHash,
				  0x0001,						  /* 1 or 5 ??? */
				  keyHandle,
				  log);
   if (ret) {
	  fprintf(stderr, "TPM_OSAP #2 returned %d\n", ret);
	  goto out;
   }


   RAND_bytes(nonceodd, TCPA_NONCE_SIZE);

   /* From seal.c: calculate KEY authorization HMAC value */
   ret = authhmac(certAuth,
				  certAuthHandle.ssecret, TCPA_HASH_SIZE,
				  certAuthHandle.enonce, nonceodd,
				  0,

				  4, &ordinal,
				  TCPA_HASH_SIZE, antiReplay,
				  
				  0, 0);
   if (ret < 0) {
	  fprintf(stderr, "authmac #1 returned %d\n", ret);
	  goto out;
   }

   ret = authhmac(keyAuth,
				  keyAuthHandle.ssecret, TCPA_HASH_SIZE,
				  keyAuthHandle.enonce, nonceodd,
				  0,

				  4, &ordinal,
				  TCPA_HASH_SIZE, antiReplay,

				  0, 0);

   if (ret != 0) {
	  fprintf(stderr, "authmac #2 returned %d\n", ret);
	  goto out;
   }


   ret = buildbuff(certify_key, tcpadata,
				   ordinal,						  /* l */
				   certHandle,					  /* L */
				   keyHandle,					  /* L */

				   TCPA_NONCE_SIZE, antiReplay,	  /* % */
				   certAuthHandle.handle, /* L */
				   TCPA_NONCE_SIZE, nonceodd,	  /* % */
				   0,							  /* o (ignored) */
				   TCPA_HASH_SIZE, certAuth,	  /* % */
				   
				   keyAuthHandle.handle, /* L */
				   TCPA_NONCE_SIZE, nonceodd,	  /* % */
				   0,							  /* o (ignored) */
				   TCPA_HASH_SIZE, keyAuth	  /* % */
				   );

   if (ret <= 0) {
	  fprintf(stderr, "buildbuff(...) = %d\n", ret);
	  goto out;
   }


   ret = TPM_Transmit(tcpadata, log, "CertifyKey");
   if (ret != 0) {
	  fprintf(stderr, "TPM_Transmit(tcpadata) = %d\n", ret);
	  goto out;
   }

   ret = ntohl(*(uint32_t *) & tcpadata[TCPA_RETURN_OFFSET]);
   if (ret != 0) {
	  fprintf(stderr, "TCPA_RESULT = %d\n", ret);
	  goto out;
   }

   
#if 0											  /* checkhmac1 fails */
   /* check the response HMAC */
   printf("KeySize returned %d\n", ret);
   
   ret = checkhmac1(tcpadata, ordinal,
					nonceodd,
					certAuthHandle.ssecret, TCPA_HASH_SIZE,

					size????, TCPA_DATA_OFFSET,
					
					0, 0);
   
   if (ret != 0) {
	  fprintf(stderr, "checkhmac1 = %d\n", ret);
	  goto out;
   }
#endif
   
 out:
   if(certAuthHandle.handle != 0xffffffff) {
	  TPM_Terminate_Handle(certAuthHandle.handle, log);
   }
   
   if(keyAuthHandle.handle != 0xffffffff) {
	  TPM_Terminate_Handle(keyAuthHandle.handle, log);
   }
   
   exit(ret);
}

