/****************************************************************************/
/*                                                                          */
/*                            Seal a Data File                              */
/*                                                                          */
/*  This file is copyright 2003 IBM. See "License" for details              */
/****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <tcpa.h>

#include <buildbuff.h>

#define TPM_TAG_RQU_AUTH2_COMMAND 0x00C3
#define TPM_ORD_MakeIdentity 121
#define TPM_KEY_IDENTITY 0x0012

#define TPM_AUTH_NEVER 0x00
#define TPM_AUTH_ALWAYS 0x01

int main(int argc, char *argv[]) {
   char *ownerPassword = "tnd0007";
   unsigned char ownerPasswordHash[TCPA_HASH_SIZE];
   
   FILE *log = stdout;

   int i;

   unsigned char dir_write_auth[] = "00 C2 T l L % L % o %";

   unsigned char tcpadata[TCPA_MAX_BUFF_SIZE];

   uint32_t authHandle;
   unsigned char nonceOdd[TCPA_HASH_SIZE];
   unsigned char authLastNonceEven[TCPA_HASH_SIZE];

   unsigned char *newContents = "01234567890123456789";
   uint32_t dirIndex;
   
   unsigned char ownerAuth[TCPA_HASH_SIZE];

   uint32_t ordinal = htonl(0x19);
   
   int ret;

   authHandle = 0xffffffff;

   sha1(ownerPassword, strlen(ownerPassword), ownerPasswordHash);

   /* Open OIAP Session */
   ret = TPM_OIAP(&authHandle, authLastNonceEven, log);
   if (ret) {
	  fprintf(stderr, "TPM_OIAP returned %d\n", ret);
	  goto out;
   }

   RAND_bytes(nonceOdd, TCPA_NONCE_SIZE);

   /* From seal.c: calculate KEY authorization HMAC value */
   ret = authhmac(ownerAuth,
				  ownerPasswordHash, TCPA_HASH_SIZE,
				  authLastNonceEven, nonceOdd,
				  0,

				  sizeof(ordinal), &ordinal,
				  sizeof(dirIndex), &dirIndex,
				  TCPA_HASH_SIZE, newContents,
				  
				  0, 0);
   if (ret < 0) {
	  goto out;
   }

   ret = buildbuff(dir_write_auth, tcpadata,
				   ordinal,						  /* l */
				   0,							  /* L */
				   TCPA_HASH_SIZE, newContents,	  /* % */
				   authHandle,					  /* L */
				   TCPA_NONCE_SIZE, nonceOdd,	  /* % */
				   0,							  /* o */
				   TCPA_HASH_SIZE, ownerAuth	  /* % */
				   );

   if (ret <= 0) {
	  fprintf(stderr, "buildbuff(...) = %d\n", ret);
	  goto out;
   }


   ret = TPM_Transmit(tcpadata, log, "MakeIdentity");
   if (ret != 0) {
	  fprintf(stderr, "TPM_Transmit(tcpadata) = %d\n", ret);
	  goto out;
   }

   ret = ntohl(*(uint32_t *) & tcpadata[TCPA_RETURN_OFFSET]);
   if (ret != 0) {
	  fprintf(stderr, "TCPA_RESULT = %d\n", ret);
	  goto out;
   }

   
 out:
   if(authHandle != 0xffffffff) {
	  TPM_Terminate_Handle(authHandle, log);
   }
   
   exit(ret);
}

