/****************************************************************************/
/*                                                                          */
/*                            Seal a Data File                              */
/*                                                                          */
/*  This file is copyright 2003 IBM. See "License" for details              */
/****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <tcpa.h>

#include <openssl/rsa.h>
#include <openssl/pem.h>
#include <openssl/evp.h>

#include <buildbuff.h>

#define TPM_TAG_RQU_AUTH2_COMMAND 0x00C3
#define TPM_ORD_MakeIdentity 121
#define TPM_KEY_IDENTITY 0x0012

#define TPM_AUTH_NEVER 0x00
#define TPM_AUTH_ALWAYS 0x01

int main(int argc, char *argv[]) {
   /* Should be arguments */
   char *ownerPassword = "tnd0007";
   char *srkPassword = "tnd0007";
   unsigned char *labelPrivCADigest = "01234567890123456789";

   
   FILE *log = stdout;

   int i;

   /* TPM_TAG_RQU_AUTH2_COMMAND */
   unsigned char make_identity[] = "00 C3 T l % % % L % o % L % o %";

   unsigned char tcpadata[TCPA_MAX_BUFF_SIZE];

   unsigned char identityAuth[TCPA_HASH_SIZE];
   unsigned char xorwork[TCPA_HASH_SIZE * 2];
   unsigned char xorhash[TCPA_HASH_SIZE];


   unsigned char idKeyParams[TCPA_MAX_BUFF_SIZE];
   keydata k, q;
   int idKeyParams_len;

   /* osapsess srkAuthHandle; */
   uint32_t srkAuthHandle;
   unsigned char srknonceOdd[TCPA_HASH_SIZE];

   unsigned char srkAuth[TCPA_HASH_SIZE];
   unsigned char srkPasswordHash[TCPA_HASH_SIZE];
   unsigned char ownerPasswordHash[TCPA_HASH_SIZE];
   unsigned char nonceodd[TCPA_NONCE_SIZE];

   osapsess authHandle;
   
   unsigned char ownerAuth[TCPA_HASH_SIZE];

   uint32_t ordinal = htonl(TPM_ORD_MakeIdentity);
   
   int ret;

   /* For writing the identity key out */
   RSA *rsa;                   /* OpenSSL format Public Key */
   FILE *keyfile;              /* output file for public key */
   EVP_PKEY pkey;              /* OpenSSL public key */
   unsigned char pubkeyHash[TCPA_HASH_SIZE];
   
   authHandle.handle = 0xffffffff;
   srkAuthHandle = 0xffffffff;

   /* Build keydata k */
   memset(&k, sizeof(k), 0);

   k.version[0] = 1;							  /* version 1.1.0.0 */
   k.version[1] = 1;
   k.version[2] = 0;
   k.version[3] = 0;
   k.keyusage = TPM_KEY_IDENTITY;				  /* Identity Key */
   k.keyflags = 0;

   /* ??? */
   k.authdatausage = TPM_AUTH_ALWAYS;			  /* Key usage must be authorized */

   k.privkeylen = 0;							  /* private key not specified here */
   
   /* ??? */
   k.pub.algorithm = 0x00000001;				  /* RSA */
   k.pub.encscheme = 0x0001;					  /* NONE */
   k.pub.sigscheme = 0x0002;					  /* SHA1 */
   
   k.pub.keybitlen = RSA_MODULUS_BIT_SIZE;
   k.pub.numprimes = 2;
   k.pub.expsize = 0;							  /* defaults to 0x010001 */
   k.pub.keylength = 0;							  /* not used here */
   k.pub.pcrinfolen = 0;						  /* not used here */
   
   
   idKeyParams_len = BuildKey(idKeyParams, &k);
   if (idKeyParams_len <= 0) {
	  fprintf(stderr, "BuildKey returned %d\n", idKeyParams_len);
	  goto out;
   }

   
   sha1(srkPassword, strlen(srkPassword), srkPasswordHash);
   sha1(ownerPassword, strlen(ownerPassword), ownerPasswordHash);

   
   /* Open OIAP Session */
   ret = TPM_OIAP(&srkAuthHandle, srknonceOdd, log);
   if (ret) {
	  fprintf(stderr, "TPM_OIAP returned %d\n", ret);
	  goto out;
   }
   
   /* Open OSAP Session */
   ret = TPM_OSAP(&authHandle, ownerPasswordHash,
				  0x0002,						  /* OWNER */
				  0x40000001,					  /* OWNER */
				  log);
   if (ret) {
	  fprintf(stderr, "TPM_OSAP #2 returned %d\n", ret);
	  goto out;
   }


   /* from seal.c */
   memcpy(xorwork, authHandle.ssecret, TCPA_HASH_SIZE);
   memcpy(xorwork + TCPA_HASH_SIZE, authHandle.enonce, TCPA_HASH_SIZE);
   sha1(xorwork, TCPA_HASH_SIZE * 2, xorhash);

   /* encrypt data authorization key */
   for (i = 0; i < TCPA_HASH_SIZE; ++i)
	  identityAuth[i] = xorhash[i] ^ ownerPasswordHash[i];


   RAND_bytes(nonceodd, TCPA_NONCE_SIZE);

   /* From seal.c: calculate KEY authorization HMAC value */
   ret = authhmac(srkAuth,
				  srkPasswordHash, TCPA_HASH_SIZE,
				  srknonceOdd, nonceodd,
				  0,

				  4, &ordinal,
				  TCPA_HASH_SIZE, identityAuth,
				  TCPA_HASH_SIZE, labelPrivCADigest,
				  idKeyParams_len, idKeyParams,
				  
				  0, 0);
   if (ret < 0) {
	  fprintf(stderr, "authmac #1 returned %d\n", ret);
	  goto out;
   }

   ret = authhmac(ownerAuth,
				  authHandle.ssecret, TCPA_HASH_SIZE,
				  authHandle.enonce, nonceodd,
				  0,

				  4, &ordinal,
				  TCPA_HASH_SIZE, identityAuth,
				  TCPA_HASH_SIZE, labelPrivCADigest,
				  idKeyParams_len, idKeyParams,

				  0, 0);

   if (ret != 0) {
	  fprintf(stderr, "authmac #2 returned %d\n", ret);
	  goto out;
   }


   ret = buildbuff(make_identity, tcpadata,
				   ordinal,						  /* l */
				   TCPA_HASH_SIZE, identityAuth,  /* % */
				   TCPA_HASH_SIZE, labelPrivCADigest, /* % */
				   idKeyParams_len, idKeyParams,  /* % */
				   srkAuthHandle,				  /* L */
				   TCPA_NONCE_SIZE, nonceodd,	  /* % */
				   0,							  /* o (ignored) */
				   TCPA_HASH_SIZE, srkAuth,		  /* % */
				   authHandle.handle,			  /* L */
				   TCPA_NONCE_SIZE, nonceodd,	  /* % */
				   0,							  /* o (ignored) */
				   TCPA_HASH_SIZE, ownerAuth	  /* % */
				   );

   if (ret <= 0) {
	  fprintf(stderr, "buildbuff(...) = %d\n", ret);
	  goto out;
   }


   ret = TPM_Transmit(tcpadata, log, "MakeIdentity");
   if (ret != 0) {
	  fprintf(stderr, "TPM_Transmit(tcpadata) = %d\n", ret);
	  goto out;
   }

   ret = ntohl(*(uint32_t *) & tcpadata[TCPA_RETURN_OFFSET]);
   if (ret != 0) {
	  fprintf(stderr, "TCPA_RESULT = %d\n", ret);
	  goto out;
   }

   
   KeyExtract(tcpadata + TCPA_DATA_OFFSET, &q);

#if 0											  /* checkhmac1 fails */
   /* check the response HMAC */
   ret = KeySize(tcpadata + TCPA_DATA_OFFSET);
   printf("KeySize returned %d\n", ret);
   
   ret = checkhmac1(tcpadata, ordinal,
					nonceodd,
					authHandle.ssecret, TCPA_HASH_SIZE,

					ret, TCPA_DATA_OFFSET,

					0, 0);
   
   if (ret != 0) {
	  fprintf(stderr, "checkhmac1 = %d\n", ret);
	  goto out;
   }
#endif
   

    printf("Created key with name identity.key\n");

    /* write out keydata struct for future TPM_LoadKey calls */
    keyfile = fopen("identity.key", "w");
    fwrite(&q, sizeof(q), 1, keyfile);
    fclose(keyfile);

	/*
	** convert the returned public key to OpenSSL format and
	** export it to a file
	*/
    rsa = convpubkey(&(q.pub));
    if (rsa == NULL) {
        fprintf(stderr, "Error from TSS_convpubkey\n");
		ret = 3;
		goto out;
    }
    OpenSSL_add_all_algorithms();
    EVP_PKEY_assign_RSA(&pkey, rsa);

    keyfile = fopen("identity.pem", "w");

	if (keyfile == NULL) {
        fprintf(stderr, "Unable to create public key file\n");
		ret = 4;
		goto out;
    }

    ret = PEM_write_PUBKEY(keyfile, &pkey);
    if (ret == 0) {
        fprintf(stderr, "Unable to write public key file\n");
		ret = 5;
		goto out;
    }
	ret = 0;					/* PEM_write_PUBKEY succeeded */

    fclose(keyfile);
    RSA_free(rsa);

	keyfile = fopen("identity.pub.sha1", "w");
	sha1(q.pub.modulus, q.pub.keylength, pubkeyHash);
	fwrite(pubkeyHash, TCPA_HASH_SIZE, 1, keyfile);

 out:
   if(srkAuthHandle != 0xffffffff) {
	  TPM_Terminate_Handle(srkAuthHandle, log);
   }
   
   if(authHandle.handle != 0xffffffff) {
	  TPM_Terminate_Handle(authHandle.handle, log);
   }
   
   exit(ret);
}

